<?php

namespace App\Http\Controllers;

use App\Mappings\BetCodeMapping;
use App\Models\Game;
use App\Models\User;
use App\Services\GameService;
use App\Services\UserService;
use Illuminate\Http\Request;
use Illuminate\Validation\ValidationException;

class AuthController extends Controller
{
    public function __construct(
        protected UserService    $userService,
        protected GameService    $gameService,
        protected BetCodeMapping $betCodeMapping
    )
    {
    }

    public function index()
    {
        if (auth()->check()) {
            return redirect()
                ->route('two-factor.index');
        }

        return view('pages.game.auth');
    }

    public function store(Request $request)
    {
        $errors = [];

        $game = $this->gameService
            ->findBySubdomain(getSubdomain());
        abort_if(is_null($game), 404);

        $betCodeInfo = $this->betCodeMapping::generateBetCodeInfo($request->bet_code);

        $user = $this->userService
            ->findByBetCodeAndGame($game, $betCodeInfo['bet_code']);

        $userByNif = $this->userService
            ->findByNifAndGame($game, $betCodeInfo['nif_3']);

        $this->validateIp(
            $request->ip(),
            $game,
            $user,
            $request->has('force_ip')
        );

        if (is_null($user) && is_null($userByNif)) {
            $user = $this->userService
                ->store(
                    $game,
                    $betCodeInfo['name'],
                    $request->email,
                    $betCodeInfo['bet_code'],
                    $betCodeInfo['nif_3'],
                    $betCodeInfo['city'],
                    $request->ip()
                );
        }

        if (is_null($user) && !is_null($userByNif)) {
            $errors[] = ['userActions' => __('auth.nif_repeated ')];
        }

        $currentActions = optional(optional($user)->confirmedUserActions)->count() ?? 0;
        if ($currentActions >= 3) {
            $errors[] = ['userActions' => __('api-response.prize-out-of-tries')];
        }

        if (!empty($errors)) {
            return back()->withErrors($errors);
        }

        auth()->login($user);

        $this->userService->sendTwoFactorAuthCode($user);

        return redirect()
            ->route('two-factor.index');
    }

    private function validateIp($ip, Game $game, ?User $user, $forceIp = false): void
    {
        if (!$forceIp) {
            $usersOnIp = $this->userService->getByIpAndGame($game, $ip);
            if ($usersOnIp->isNotEmpty() && is_null($usersOnIp->where('id', $user?->id)->first())) {
                throw ValidationException::withMessages(['ip' => __('auth.auth_ip')]);
            }
        }
    }

    public function logout()
    {
        $this->userService->logoutAndClearSession();

        return redirect()->route('auth.index');
    }
}
