<?php

namespace App\Http\Controllers;

use App\Http\Requests\PrizeRequest;
use App\Mail\GameWinMail;
use App\Mappings\GameActionModalMapping;
use App\Mappings\GameMapping;
use App\Models\UserAction;
use App\Services\GameService;
use App\Services\LottoService;
use App\Services\UserActionService;
use App\Services\UserService;
use Illuminate\Http\JsonResponse;
use Illuminate\Support\Facades\Mail;
use Symfony\Component\HttpFoundation\Response as ResponseAlias;
use Throwable;

class PrizeController extends Controller
{
    public function __construct(
        protected UserService       $userService,
        protected UserActionService $userActionService,
        protected GameService       $gameService,
        protected GameMapping       $gameMapping
    )
    {
    }

    public function prize(PrizeRequest $request, LottoService $gameService)
    {
        $user = auth()->user();
        $game = $this->gameService->findBySubdomain(getSubdomain());

        if ($user->confirmedUserActions->count() >= $game->rounds) {
            auth()->logout();
            return response()->json(
                [
                    'redirect' => route('auth.index'),
                    'message' => [__('api-response.prize-out-of-tries')],
                ],
                ResponseAlias::HTTP_BAD_REQUEST
            );
        }

        $action = $request->action;

        $orangeBalls = collect(array_map('intval', $request->number_orange ?? []));
        $greenBalls = collect(array_map('intval', $request->number_green ?? []));
        $purpleBalls = collect(array_map('intval', $request->number_purple ?? []));

        /**
         * can only predict between 1 and 6 balls in total
         * if one of the selected balls is purple, user can not select green balls
         */
        if (!GameMapping::getSelectedBallsValid($orangeBalls, $greenBalls, $purpleBalls)) {
            return response()->json(
                [
                    'message' => [__('api-response.prize-fix-error')],
                ],
                ResponseAlias::HTTP_BAD_REQUEST
            );
        }

        $result = $gameService->prize($orangeBalls, $greenBalls, $purpleBalls, $action);

        if ($result['error']) {
            return response()->json(
                [
                    'message' => [__('api-response.prize-fix-error')],
                ],
                ResponseAlias::HTTP_OK
            );
        }

        // store user action
        $userAction = $this->userActionService
            ->store(
                auth()->user(),
                $result['orange_take'],
                $result['green_take'],
                $result['purple_take'],
                $result['orange_prize'],
                $result['green_prize'],
                $result['purple_prize'],
                $action === 'fix-sort',
                $result['win'],
                false
            );

        if ($action === 'fix-sort') {
            $data = ['fix-sort' => true];
        } else {
            $data = [
                'prize_orange' => $result['orange_prize']->implode(' '),
                'prize_green' => $result['green_prize']->implode(' '),
                'prize_purple' => $result['purple_prize']->implode(' '),
            ];
        }

        if (!$result['win'] && config('game.ALWAYS_WIN')) {
            $result['win'] = true;
        }

        if ($result['win']) {
            Mail::to(config('mail.from.address'))->send(new GameWinMail($userAction));
            Mail::to($user->email)->send(new GameWinMail($userAction));
        }

        $user->refresh();
        $playsRemaining = $game->rounds - $user->confirmedUserActions->count() - 1;
        $currentTrial = $this->gameMapping::getCurrentTrial($user, $game) + 1;
        $confirmedUserActions = $user->confirmedUserActions->push($userAction);

        return response()->json(
            [
                'prizeId' => $userAction->id,
                'currentTrial' => $currentTrial,
                'playsRemaining' => $playsRemaining,
                'data' => $data,
                'message' => GameActionModalMapping::getGameActionModalMessage($playsRemaining, $confirmedUserActions, $result['win']),
                'resultsMessage' => GameActionModalMapping::getResultsModalMessage($confirmedUserActions, $user),
                'display_message' => $action !== 'only-sort',
            ],
            ResponseAlias::HTTP_OK
        );
    }

    /**
     * @param UserAction $userAction
     *
     * @return JsonResponse
     */
    public function confirmPrize(UserAction $userAction)
    {
        $user = auth()->user();
        $game = $this->gameService->findBySubdomain(getSubdomain());

        if ($user->confirmedUserActions->count() >= $game->rounds) {
            auth()->logout();
            return response()->json(
                [
                    'redirect' => route('auth.index'),
                    'message' => [__('api-response.prize-out-of-tries')],
                ],
                ResponseAlias::HTTP_BAD_REQUEST
            );
        }

        if ($userAction->user_id !== $user->id) {
            return response()->json(
                [
                    'message' => [__('api-response.prize-does-not-belong-to-you')],
                ],
                ResponseAlias::HTTP_FORBIDDEN
            );
        }

        $this->userActionService->confirm($userAction);

        $user->refresh();
        $playsRemaining = $game->rounds - $user->confirmedUserActions->count();

        if ($playsRemaining <= 0) {
            $this->userService->logoutAndClearSession();
        }

        return response()->json([], ResponseAlias::HTTP_OK);
    }

    /**
     *
     * @return array|JsonResponse
     */
    public function listPrize()
    {
        $user = auth()->user();

        try {
            $data = $user->userActions;
            return response()->json(
                [
                    'data' => $data,
                    'message' => [__('api-response.prize-load-success')],
                ],
                ResponseAlias::HTTP_OK
            );
        } catch (Throwable $e) {
            report($e);
            return response()->json(
                [
                    'message' => ['Error: list prize.'],
                ],
                ResponseAlias::HTTP_BAD_GATEWAY
            );
        }
    }

    /**
     *
     * @return array|JsonResponse
     */
    public function showResults()
    {
        $user = auth()->user();

        try {
            return response()->json(
                [
                    'data' => $user->confirmedUserActions,
                    'message' => GameActionModalMapping::getResultsModalMessage($user->confirmedUserActions, $user),
                ],
                ResponseAlias::HTTP_OK
            );
        } catch (Throwable $e) {
            report($e);
            return response()->json(
                [
                    'message' => ['Error: show results.'],
                ],
                ResponseAlias::HTTP_BAD_GATEWAY
            );
        }
    }
}
