<?php

namespace App\Http\Requests;

use Illuminate\Foundation\Http\FormRequest;

class PrizeRequest extends FormRequest
{
    protected int $max_orange = 5;
    protected int $min_orange = 1;
    protected int $max_green = 1;
    protected int $max_purple = 1;
    protected int $min_number_ball = 1;
    protected int $max_number_ball = 16;

    /**
     * Determine if the user is authorized to make this request.
     */
    public function authorize(): bool
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array<string, \Illuminate\Contracts\Validation\Rule|array|string>
     */
    public function rules(): array
    {
        /**
         * fix-sort -> will use a fixed sort (prize balls) from table fix_prizes and inform if the user won
         * only-sort -> will only return a random set of sort (prize balls)
         * random-sort ->  will return a random set of sort (prize balls) and inform if the user won
         * stick-sort -> same as the option random-sort
         */
        $action = $this->action;

        if (isset($action) && $action === 'only-sort') {
            return [
                'number_orange' => 'array|min:' . $this->min_orange . '|max:' . $this->max_orange,
                'number_green' => 'array|max:' . $this->max_green,
                'number_purple' => 'array|max:' . $this->max_purple,
            ];
        } else {
            return [
                'action' => 'required|in:only-sort,random-sort,fix-sort,stick-sort',
                'number_orange' => 'required|array|min:' . $this->min_orange . '|max:' . $this->max_orange,
                'number_orange.*' =>
                    'required|distinct|min:' . $this->min_number_ball . '|max:' . $this->max_number_ball . '|numeric',
                'number_green' => 'array|max:' . $this->max_green,
                'number_green.*' =>
                    'required|distinct|min:' . $this->min_number_ball . '|max:' . $this->max_number_ball . '|numeric',
                'number_purple' => 'array|max:' . $this->max_purple,
                'number_purple.*' =>
                    'required|distinct|min:' . $this->min_number_ball . '|max:' . $this->max_number_ball . '|numeric',
            ];
        }
    }

    public function messages(): array
    {
        return [
            'action.required' => 'Please select one of the options bellow, blue buttons.',
            'action.in' => 'Wrong action.',
            'number_orange.required' => 'Please select orange balls.',
            'number_orange.*.numeric' =>
                'Please select numbers between ' . $this->min_number_ball . ' and ' . $this->max_number_ball . '.',
            'number_orange.*.min' =>
                'Please select numbers between ' . $this->min_number_ball . ' and ' . $this->max_number_ball . '.',
            'number_orange.*.max' =>
                'Please select numbers between ' . $this->min_number_ball . ' and ' . $this->max_number_ball . '.',
            'number_orange.min' => 'Please select at least ' . $this->min_orange . ' orange balls.',
            'number_orange.max' => 'You cannot select more than ' . $this->max_orange . ' orange balls.',
            'number_orange.*.distinct' => 'You cannot repeat numbers balls.',

            'number_green.required' =>
                'Please select numbers between ' . $this->min_number_ball . ' and ' . $this->max_number_ball . '.',
            'number_green.*.numeric' =>
                'Please select numbers between ' . $this->min_number_ball . ' and ' . $this->max_number_ball . '.',
            'number_green.*.min' =>
                'Please select numbers between ' . $this->min_number_ball . ' and ' . $this->max_number_ball . '.',
            'number_green.*.max' =>
                'Please select numbers between ' . $this->min_number_ball . ' and ' . $this->max_number_ball . '.',
            'number_green.max' => 'You cannot select more than ' . $this->max_green . ' green balls.',
            'number_green.*.distinct' => 'You cannot repeat numbers balls.',

            'number_purple.required' =>
                'Please select numbers between ' . $this->min_number_ball . ' and ' . $this->max_number_ball . '.',
            'number_purple.*.numeric' =>
                'Please select numbers between ' . $this->min_number_ball . ' and ' . $this->max_number_ball . '.',
            'number_purple.*.min' =>
                'Please select numbers between ' . $this->min_number_ball . ' and ' . $this->max_number_ball . '.',
            'number_purple.*.max' =>
                'Please select numbers between ' . $this->min_number_ball . ' and ' . $this->max_number_ball . '.',
            'number_purple.max' => 'You cannot select more than ' . $this->max_purple . ' purple balls.',
            'number_purple.*.distinct' => 'You cannot repeat numbers balls.',
        ];
    }
}
